import numpy as np
from osgeo import gdal
import os
from glob import glob
from rindcalc.utils import save_index, resample


class Sentinel:

    def __init__(self, path):
        """
        Class to read and write Sentinel-2 data from.

        Parameters
        ----------
            path : str
                Path to folder where Sentinel-2 bands are contained.

        Attributes
        ----------
            path : dict
                Dictionary of the path for each Sentinel-2 band.
            bands : dict, array
                Dictionary of arrays for the bands chosen to load.
            band_options : list
                List of all options for band input names.
        """
        ends = ['*B01.jp2', '*B02.jp2', '*B03.jp2', '*B04.jp2', '*B05.jp2',
                '*B06.jp2', '*B07.jp2', '*B08.jp2', '*B8A.jp2', '*B09.jp2',
                '*B10.jp2', '*B11.jp2', '*B12.jp2']

        paths = []
        for i in range(len(ends)):
            paths.append(glob(os.path.join(path, ends[i])))
        self.bands = {}
        self.path = {"band_1": paths[0][0], "band_2": paths[1][0],
                     "band_3": paths[2][0], "band_4": paths[3][0],
                     "band_5": paths[4][0], "band_6": paths[5][0],
                     "band_7": paths[6][0], "band_8": paths[7][0],
                     "band_8a": paths[8][0], "band_9": paths[9][0],
                     "band_10": paths[10][0], "band_11": paths[11][0],
                     "band_12": paths[12][0]}
        self.band_options = ['band_1', 'band_2', 'band_3', 'band_4',
                             'band_5', 'band_6', 'band_7', 'band_8',
                             'band_8a', 'band_9', 'band_10', 'band_11',
                             'band_12']

    def load_bands(self, which_bands=None):
        """
        Opens and reads bands into Float 32 arrays. If no list is passed into
        `which` bands then all bands are opened and added to the dictionary
        `self.bands`.

        Parameters
        ----------
            which_bands : list, optional
                A list of band names to open as arrays.
                e.g. which_bands=['band_1', 'band_2', 'band_3']

        Returns
        -------
            self.bands : dict
                Updated self.bands dictionary

        """

        paths = self.path
        bands = {}
        if which_bands is None:
            which_bands = self.band_options
        for i in range(len(which_bands)):
            if which_bands[i] == 'band_1':
                band = gdal.Open(paths["band_1"])
                band_1 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_1": band_1})
            if which_bands[i] == 'band_2':
                band = gdal.Open(paths["band_2"])
                band_2 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_2": band_2})
            if which_bands[i] == 'band_3':
                band = gdal.Open(paths["band_3"])
                band_3 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_3": band_3})
            if which_bands[i] == 'band_4':
                band = gdal.Open(paths["band_4"])
                band_4 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_4": band_4})
            if which_bands[i] == 'band_5':
                band = gdal.Open(paths["band_5"])
                band_5 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_5": band_5})
            if which_bands[i] == 'band_6':
                band = gdal.Open(paths["band_6"])
                band_6 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_6": band_6})
            if which_bands[i] == 'band_7':
                band = gdal.Open(paths["band_7"])
                band_7 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_7": band_7})
            if which_bands[i] == 'band_8':
                band = gdal.Open(paths["band_8"])
                band_8 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_8": band_8})
            if which_bands[i] == 'band_8a':
                band = gdal.Open(paths["band_8a"])
                band_8a = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_8a": band_8a})
            if which_bands[i] == 'band_9':
                band = gdal.Open(paths["band_9"])
                band_9 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_9": band_9})
            if which_bands[i] == 'band_10':
                band = gdal.Open(paths["band_10"])
                band_10 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_10": band_10})
            if which_bands[i] == 'band_11':
                band = gdal.Open(paths["band_11"])
                band_11 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_11": band_11})
            if which_bands[i] == 'band_12':
                band = gdal.Open(paths["band_12"])
                band_12 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_12": band_12})

        self.bands = bands
        return self.bands

    def composite(self, which_bands, out_composite):
        """
        Creates a three band composite out of the specified bands.

        Parameters
        ----------
            which_bands : list
                A list of bands to save as a three band composite. Must be in
                order of how the bands are to saved within the output TIFF.
                e.g. which_bands=['band_1', 'band_2', 'band_3']
            out_composite : str
                The output filename to save the composite,
        """

        paths = self.path
        bands = []
        for i in range(len(which_bands)):
            if which_bands[i] == 'band_1':
                bands.append(paths["band_1"])
            if which_bands[i] == 'band_2':
                bands.append(paths["band_2"])
            if which_bands[i] == 'band_3':
                bands.append(paths["band_3"])
            if which_bands[i] == 'band_4':
                bands.append(paths["band_4"])
            if which_bands[i] == 'band_5':
                bands.append(paths["band_5"])
            if which_bands[i] == 'band_6':
                bands.append(paths["band_6"])
            if which_bands[i] == 'band_7':
                bands.append(paths["band_7"])
            if which_bands[i] == 'band_8':
                bands.append(paths["band_8"])
            if which_bands[i] == 'band_8a':
                bands.append(paths["band_8"])
            if which_bands[i] == 'band_9':
                bands.append(paths["band_9"])
            if which_bands[i] == 'band_10':
                bands.append(paths["band_10"])
            if which_bands[i] == 'band_11':
                bands.append(paths["band_11"])
            if which_bands[i] == 'band_12':
                bands.append(paths["band_12"])

        vrt = gdal.BuildVRT('tmp.vrt', bands, separate=True, bandList=[1, 1, 1])
        trans = gdal.Translate(out_composite, vrt, format='GTiff')

    def AWEIsh(self, out_raster=None):
        """
        Calculates AWEIsh index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """

        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(["band_2", "band_3", "band_8"])

        band_11 = resample(self.path['band_11'], 10)
        band_12 = resample(self.path['band_12'], 10)

        a = (bands["band_2"] + 2.5 * bands["band_3"] - 1.5 *
             (bands["band_8"] + band_11) - 0.25 *
             band_12)
        b = (bands["band_2"] + bands["band_3"] + bands["band_8"]
             + band_11 + band_12)
        equation = a / b

        snap = self.path['band_2']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def NDVI(self, out_raster=None):
        """
        Calculates NDVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """

        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(["band_4", "band_8"])

        equation = (bands["band_8"] - bands["band_4"]) / \
                   (bands["band_8"] + bands["band_4"])

        snap = self.path['band_2']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def SIPI(self, out_raster=None):
        """
        Calculates SIPI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(["band_2", "band_8"])

        equation = (bands["band_8"] - bands["band_2"]) / \
                   (bands["band_8"] + bands["band_2"])

        snap = self.path['band_2']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def ARVI(self, out_raster=None):
        """
        Calculates ARVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(["band_2", "band_4", "band_8"])

        equation = ((bands["band_8"] - (2 * bands["band_4"]) + bands["band_2"])/
                    (bands["band_8"] + (2 * bands["band_4"]) + bands["band_2"]))

        snap = self.path['band_2']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def NDI45(self, out_raster=None):
        """
        Calculates NDI45 index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_4'])
        band_5 = resample(self.path['band_5'], 10)

        equation = ((band_5 - bands['band_4']) /
                    (band_5 + bands['band_4']))

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def MTCI(self, out_raster=None):
        """
        Calculates MTCI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_4'])
        band_5 = resample(self.path['band_5'], 10)
        band_6 = resample(self.path['band_6'], 10)

        equation = ((band_6 - band_5) /
                    (band_5 - bands['band_4']))

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def MCARI(self, out_raster=None):
        """
        Calculates MCARI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_3', 'band_4'])
        band_5 = resample(self.path['band_5'], 10)

        equation = (abs((band_5 - bands['band_4']) - 0.2 *
                    (band_5 - bands['band_3'])) *
                    (band_5 - bands['band_4']))

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def GNDVI(self, out_raster=None):
        """
        Calculates GNDVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_3'])
        band_7 = resample(self.path['band_7'], 10)

        equation = ((band_7 - bands['band_3']) /
                    (band_7 + bands['band_3']))

        snap = self.path['band_3']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def PSSR(self, out_raster=None):
        """
        Calculates PSSR index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        band_7 = resample(self.path['band_7'], 10)
        bands = self.load_bands(['band_4'])

        equation = (band_7 / bands['band_4'])

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def S2REP(self, out_raster=None):
        """
        Calculates S2REP index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_4'])
        band_5 = resample(self.path['band_5'], 10)
        band_6 = resample(self.path['band_6'], 10)
        band_7 = resample(self.path['band_7'], 10)

        equation = 705 + 35 * ((((band_7 + bands['band_4']) / 2) - band_5) /
                               (band_6 - band_5))

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def IRECI(self, out_raster=None):
        """
        Calculates IRECI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_4'])
        band_5 = resample(self.path['band_5'], 10)
        band_6 = resample(self.path['band_6'], 10)
        band_7 = resample(self.path['band_7'], 10)

        equation = ((band_7 - bands['band_4']) / (band_5 / band_6))

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation

    def SAVI(self, soil_moisture=0.5, out_raster=None):
        """
        Calculates SAVI index

        Parameters
        ----------
            soil_moisture : flt
                Soil moisture value, default = 0.5
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_4', 'band_8'])

        equation = (((bands['band_8'] - bands['band_4']) / (bands['band_8'] +
                      bands['band_4'] + soil_moisture)) * (1 + soil_moisture))

        snap = self.path['band_4']

        if out_raster is not None:
            save_index(equation, out_raster, snap, gdal.GDT_Float32)
        return equation






from rindcalc.utils import save_index, resample
from rindcalc import Sentinel
import pytest
import sys
import numpy as np
sys.path.append('../rindcalc')
path = './tests/data/sent'

def test_save():
    data = Sentinel(path)
    bands = data.load_bands(['band_1'])

    save_index(bands['band_1'], './tests/data/save_index.tif', data.path['band_1'])

def test_resample():
    data = Sentinel(path)

    re_arr = resample(data.path['band_1'], 1, './tests/data/resample_test.tif')



import os
from glob import glob
import numpy as np
from osgeo import gdal
from rindcalc.utils import save_index


def cloud_mask(qa_band, array):
    """
    This function masks clouds in Landsat-8 imagery using the QA band and
    returns an array to save as a raster.
    """

    gdal.UseExceptions()
    gdal.AllRegister()
    np.seterr(divide='ignore', invalid='ignore')
    qa_path = gdal.Open(qa_band)
    qa_band = qa_path.GetRasterBand(1).ReadAsArray().astype(np.uint32)

    mask_values = [2800, 2804, 2808, 2812, 6986, 6900, 6904, 6908,
                   2976, 2980, 2984, 2988, 3008, 3012, 3016, 3020,
                   7072, 7076, 7080, 7084, 7104, 7108, 7112, 7116]
    np.seterr(divide='ignore', invalid='ignore')
    m = np.ma.array(qa_band,
                    mask=np.logical_or.reduce([qa_band == value
                                               for value in mask_values]))
    np.ma.set_fill_value(m, 0)
    m1 = m.filled()
    m1[m1 != 0] = 1

    m1.reshape(qa_band.shape)
    mask_array = array * m1
    mask_array[mask_array == 0] = np.nan

    return mask_array


def save_ls(qa_band, equation, out_raster, snap, mask_clouds):
    if out_raster is not None and mask_clouds:
        masked = cloud_mask(qa_band, equation)
        save_index(masked, out_raster, snap)
        return masked
    if out_raster is not None and not mask_clouds:
        save_index(equation, out_raster, snap)
        return equation
    if out_raster is None and mask_clouds:
        masked = cloud_mask(qa_band, equation)
        return masked
    if out_raster is None and not mask_clouds:
        return equation


class Landsat:

    def __init__(self, path):
        """
        Class to read and write Landsat-8 data from.


        Parameters
        ----------
            path : str
                Path to folder where Landsat-8 bands are contained.

        Attributes
        ----------
            path : dict
                Dictionary of the path for each Landsat-8 band.
            bands : dict, array
                Dictionary of arrays for the bands chosen to load.
            band_options : list
                List of all options for band input names.
        """
        ends = ['*B1.TIF', '*B2.TIF', '*B3.TIF', '*B4.TIF', '*B5.TIF',
                '*B6.TIF', '*B7.TIF', '*B8.TIF', '*B9.TIF', '*B10.TIF',
                '*BQA.TIF']

        paths = []
        for i in range(len(ends)):
            paths.append(glob(os.path.join(path, ends[i])))

        self.bands = {}
        self.path = {"band_1": paths[0][0], "band_2": paths[1][0],
                     "band_3": paths[2][0], "band_4": paths[3][0],
                     "band_5": paths[4][0], "band_6": paths[5][0],
                     "band_7": paths[6][0], "band_8": paths[7][0],
                     "band_9": paths[8][0], "band_10": paths[9][0],
                     "band_qa": paths[10][0]}
        self.band_options = ['band_1', 'band_2', 'band_3', 'band_4',
                             'band_5', 'band_6', 'band_7', 'band_8',
                             'band_9', 'band_10', 'band_qa']

    def load_bands(self, which_bands=None):
        """
        Opens and reads bands into Float 32 arrays. If no list is passed into
        `which` bands then all bands are opened and added to the dictionary
        `self.bands`.

        Parameters
        ----------
            which_bands : list, optional
                A list of band names to open as arrays.
                e.g. which_bands=['band_1', 'band_2', 'band_3']

        Returns
        -------
            self.bands : dict
                Updated self.bands dictionary

        """

        paths = self.path
        bands = {}
        if which_bands is None:
            which_bands = self.band_options
        for i in range(len(which_bands)):
            if which_bands[i] == 'band_1':
                band = gdal.Open(paths["band_1"])
                band_1 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_1": band_1})
            if which_bands[i] == 'band_2':
                band = gdal.Open(paths["band_2"])
                band_2 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_2": band_2})
            if which_bands[i] == 'band_3':
                band = gdal.Open(paths["band_3"])
                band_3 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_3": band_3})
            if which_bands[i] == 'band_4':
                band = gdal.Open(paths["band_4"])
                band_4 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_4": band_4})
            if which_bands[i] == 'band_5':
                band = gdal.Open(paths["band_5"])
                band_5 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_5": band_5})
            if which_bands[i] == 'band_6':
                band = gdal.Open(paths["band_6"])
                band_6 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_6": band_6})
            if which_bands[i] == 'band_7':
                band = gdal.Open(paths["band_7"])
                band_7 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_7": band_7})
            if which_bands[i] == 'band_8':
                band = gdal.Open(paths["band_8"])
                band_8 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_8": band_8})
            if which_bands[i] == 'band_9':
                band = gdal.Open(paths["band_9"])
                band_9 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_9": band_9})
            if which_bands[i] == 'band_10':
                band = gdal.Open(paths["band_10"])
                band_10 = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_10": band_10})
            if which_bands[i] == 'band_qa':
                band = gdal.Open(paths["band_qa"])
                band_qa = band.GetRasterBand(1).ReadAsArray().astype(
                    np.float32)
                bands.update({"band_qa": band_qa})

        self.bands = bands
        return self.bands

    def composite(self, which_bands, out_composite):
        """
        Creates a three band composite out of the specified bands.

        Parameters
        ----------
            which_bands : list
                A list of bands to save as a three band composite. Must be in
                order of how the bands are to saved within the output TIFF.
                e.g. which_bands=['band_1', 'band_2', 'band_3']
            out_composite : str
                The output filename to save the composite,
        """

        paths = self.path
        bands = []
        for i in range(len(which_bands)):
            if which_bands[i] == 'band_1':
                bands.append(paths["band_1"])
            if which_bands[i] == 'band_2':
                bands.append(paths["band_2"])
            if which_bands[i] == 'band_3':
                bands.append(paths["band_3"])
            if which_bands[i] == 'band_4':
                bands.append(paths["band_4"])
            if which_bands[i] == 'band_5':
                bands.append(paths["band_5"])
            if which_bands[i] == 'band_6':
                bands.append(paths["band_6"])
            if which_bands[i] == 'band_7':
                bands.append(paths["band_7"])
            if which_bands[i] == 'band_8':
                bands.append(paths["band_8"])
            if which_bands[i] == 'band_9':
                bands.append(paths["band_9"])
            if which_bands[i] == 'band_10':
                bands.append(paths["band_10"])

        vrt = gdal.BuildVRT('tmp.vrt', bands, separate=True, bandList=[1, 1, 1])
        trans = gdal.Translate(out_composite, vrt, format='GTiff')

    def NDVI(self, out_raster=None, mask_clouds=False):
        """
        Calculates NDVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_3', 'band_6'])

        equation = ((bands["band_3"] - bands["band_6"]) /
                    (bands["band_3"] + bands["band_6"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)

        return out_ras

    def AWEIsh(self, out_raster=None, mask_clouds=False):
        """"
        Calculates AWEIsh index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """

        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_2', 'band_3', 'band_5', 'band_6',
                                'band_7'])

        a = (bands["band_2"] + 2.5 * bands["band_3"] - 1.5 *
             (bands["band_5"] + bands["band_6"]) - 0.25 * bands["band_7"])
        b = (bands["band_2"] + bands["band_2"] + bands["band_5"] + bands[
            "band_6"]
             + bands["band_7"])
        equation = (a / b)

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)

        return out_ras

    def AWEInsh(self, out_raster=None, mask_clouds=False):
        """
        Calculates AWEInsh index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_3', 'band_5', 'band_6',
                                'band_7'])

        equation = ((4 * (bands["band_3"] - bands["band_6"]) -
                     (0.25 * bands["band_5"] + 2.75 * bands["band_6"])) /
                    (bands["band_3"] + bands["band_6"] + bands["band_5"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)

        return out_ras

    def NDMI(self, out_raster=None, mask_clouds=False):
        """
        Calculates NDMI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_6'])

        equation = ((bands["band_5"] - bands["band_6"]) /
                    (bands["band_5"] + bands["band_6"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def MNDWI(self, out_raster=None, mask_clouds=False):
        """
        Calculates MNDWI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_3', 'band_6'])

        equation = ((bands["band_3"] - bands["band_6"]) /
                    (bands["band_3"] + bands["band_6"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def GNDVI(self, out_raster=None, mask_clouds=False):
        """
        Calculates GNDVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_3'])

        equation = ((bands["band_5"] - bands["band_3"])
                    / (bands["band_5"] + bands["band_3"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def SAVI(self, out_raster=None, soil_brightness=0.5, mask_clouds=False):
        """
        Calculates SAVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            soil_brightness : float
                Soil brightness factor to compute SAVI with. Defaults to 0.5
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_4'])

        equation = ((bands["band_5"] - bands["band_4"]) /
                    (bands["band_5"] + bands["band_4"] + soil_brightness)) \
                    * (1 + soil_brightness)

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)

        return out_ras

    def ARVI(self, out_raster=None, mask_clouds=False):
        """
        Calculates ARVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_2', 'band_4', 'band_5'])

        equation = ((bands["band_5"] - (2 * bands["band_4"]) + bands[
            "band_2"]) /
                    (bands["band_5"] + (2 * bands["band_4"]) + bands["band_2"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)

        return out_ras

    def VARI(self, out_raster=None, mask_clouds=False):
        """
        Calculates VARI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_2', 'band_3', 'band_4'])

        equation = ((bands["band_3"] - bands["band_4"]) /
                    (bands["band_3"] + bands["band_4"] - bands["band_2"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)

        return out_ras

    def NDBI(self, out_raster=None, mask_clouds=False):
        """
        Calculates NDBI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_6'])

        # Perform Calculation
        equation = ((bands["band_6"] - bands["band_5"]) / (
                    bands["band_6"] + bands["band_5"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def NDBaI(self, out_raster=None, mask_clouds=False):
        """
        Calculates NDBaI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_6', 'band_10'])

        # Perform Calculation
        equation = ((bands["band_6"] - bands["band_10"]) /
                    (bands["band_6"] + bands["band_10"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def NBLI(self, out_raster=None, mask_clouds=False):
        """
        Calculates NBLI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_4', 'band_10'])

        equation = ((bands["band_4"] - bands["band_10"]) /
                    (bands["band_4"] + bands["band_10"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def EBBI(self, out_raster=None, mask_clouds=False):
        """
        Calculates EBBI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_6', 'band_7', 'band_10'])

        ebbi = ((bands["band_6"] - bands["band_5"]) /
                (10 * ((bands["band_6"] + bands["band_10"]) ** 0.5)))
        ebbi[np.isneginf(ebbi)] = 0
        ebbi_mask = np.ma.MaskedArray(ebbi, mask=(ebbi == 0))
        ebbi_mask.reshape(ebbi.shape)

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], ebbi_mask, out_raster,
                          snap, mask_clouds)
        return out_ras

    def UI(self, out_raster=None, mask_clouds=False):
        """
        Calculates UI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_7'])

        # Perform Calculation
        equation = ((bands["band_7"] - bands["band_5"]) /
                    (bands["band_7"] + bands["band_5"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras

    def NBRI(self, out_raster=None, mask_clouds=False):
        """
        Calculates NBRI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.
            mask_clouds : bool, optional
                Whether or not to apply cloud masking to output index with
                the QA band.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_5', 'band_7'])

        # calculation
        equation = ((bands["band_5"] - bands["band_7"]) /
                    (bands["band_5"] + bands["band_7"]))

        snap = self.path['band_2']
        out_ras = save_ls(self.path['band_qa'], equation, out_raster,
                          snap, mask_clouds)
        return out_ras



import numpy as np
from osgeo import gdal


def resample(band, cell_size, out=None):
    """
    Utility function to resample a raster and output it as either an
    array or TIFF

    Parameters
    ----------
        band : str
            Path to raster file to resample
        cell_size : int
            New size of the cells
        out: str, optional
            Filename to save the output TIFF

    Returns:
        resampled array

    """

    vrt = gdal.Translate('/vsimem/mem.vrt', band, xRes=cell_size, yRes=cell_size,
                         format='VRT')
    ds = gdal.Open('/vsimem/mem.vrt')
    array = ds.ReadAsArray().astype(np.float32)

    gdal.Unlink('/vsimem/mem.vrt')

    if out is not None:
        trans = gdal.Translate(out, vrt, format='GTiff')

    return array


import os
from osgeo import gdal
import numpy as np


def save_index(in_array, out, snap, dType=gdal.GDT_Float32):
    """
    save_raster(in_array, out, snap, dType=gdal.GDT_Float32)

    Saves the input NumPy array as a one band raster.

    Parameters:

            in_array :: array, required
                * NumPy array to be saved as TIFF raster file.

            out :: str, required
                * Output path and file name for TIFF raster file.

            snap :: gdal raster, required
                * Raster file with which projections and geotransformations
                  are based off.

            dType :: gdal datatype, required (default=gdal.GDT_Float32)_
                * Datatype to save raster as.
    """
    if os.path.exists(out):
        os.remove(out)

    snap = gdal.Open(snap)

    driver = gdal.GetDriverByName('GTiff')
    metadata = driver.GetMetadata()
    shape = in_array.shape
    dst_ds = driver.Create(out,
                           xsize=shape[1],
                           ysize=shape[0],
                           bands=1,
                           eType=dType)
    proj = snap.GetProjection()
    geo = snap.GetGeoTransform()
    dst_ds.SetGeoTransform(geo)
    dst_ds.SetProjection(proj)
    dst_ds.GetRasterBand(1).WriteArray(in_array)
    dst_ds.FlushCache()
    dst_ds = None

    return in_array



import os
from glob import glob
import numpy as np
from osgeo import gdal
from rindcalc.utils import save_index


def save_comp(bands, out, snap, dType=gdal.GDT_Byte):

    if not os.path.exists(out):
        print('Writing raster')

    if os.path.exists(out):
        os.remove(out)
        print('Overwriting raster.')

    driver = gdal.GetDriverByName('GTiff')
    metadata = driver.GetMetadata()
    shape = bands[0].shape
    dst_ds = driver.Create(out,
                           xsize=shape[1],
                           ysize=shape[0],
                           bands=3,
                           eType=dType)
    proj = snap.GetProjection()
    geo = snap.GetGeoTransform()
    dst_ds.SetGeoTransform(geo)
    dst_ds.SetProjection(proj)
    dst_ds.GetRasterBand(1).WriteArray(bands[0])
    dst_ds.GetRasterBand(2).WriteArray(bands[1])
    dst_ds.GetRasterBand(3).WriteArray(bands[2])
    dst_ds.FlushCache()
    dst_ds = None


class NAIP:

    def __init__(self, path):
        """
        Class to read and write NAIP data from.

        Parameters
        ----------
            path : str
                Path to folder where Sentinel-2 bands are contained.

        Attributes
        ----------
            path : dict
                Dictionary of the path for each Landsat-8 band.
            bands : dict, array
                Dictionary of arrays for the bands chosen to load.
            band_options : list
                List of all options for band input names.
        """
        self.bands = {}
        self.path = path

        self.band_options = ['band_1', 'band_2',
                             'band_3','band_4']

    def load_bands(self, which_bands=None):
        """
        Opens and reads bands into Float 32 arrays. If no list is passed into
        `which` bands then all bands are opened and added to the dictionary
        `self.bands`.

        Parameters
        ----------
            which_bands : list, optional
                A list of band names to open as arrays.
                e.g. which_bands=['band_1', 'band_2', 'band_3']

        Returns
        -------
            self.bands : dict
                Updated self.bands dictionary

        """
        bands = {}
        naip = gdal.Open(self.path)
        if which_bands is None:
            which_bands = self.band_options
        for i in range(len(which_bands)):
            if which_bands[i] == 'band_1':
                band_1 = naip.GetRasterBand(1).ReadAsArray().astype(np.float32)
                bands.update({"band_1": band_1})
            if which_bands[i] == 'band_2':
                band_2 = naip.GetRasterBand(2).ReadAsArray().astype(np.float32)
                bands.update({"band_2": band_2})
            if which_bands[i] == 'band_3':
                band_3 = naip.GetRasterBand(3).ReadAsArray().astype(np.float32)
                bands.update({"band_3": band_3})
            if which_bands[i] == 'band_4':
                band_4 = naip.GetRasterBand(4).ReadAsArray().astype(np.float32)
                bands.update({"band_4": band_4})

        snap = self.path
        bands.update({"snap": snap})

        self.bands = bands
        return self.bands

    def composite(self, which_bands, out_composite):
        """
        Creates a three band composite out of the specified bands.

        Parameters
        ----------
             which_bands : list
                 A list of bands to save as a three band composite. Must be in
                 order of how the bands are to saved within the output TIFF.
                 e.g. which_bands=['band_1', 'band_2', 'band_3']
             out_composite : str
                 The output filename to save the composite,
        """
        bands = []
        naip = gdal.Open(self.path)
        for i in range(len(which_bands)):
            if which_bands[i] == 'band_1':
                band_1 = naip.GetRasterBand(1).ReadAsArray().astype(np.float32)
                bands.append(band_1)
            if which_bands[i] == 'band_2':
                band_2 = naip.GetRasterBand(2).ReadAsArray().astype(np.float32)
                bands.append(band_2)
            if which_bands[i] == 'band_3':
                band_3 = naip.GetRasterBand(3).ReadAsArray().astype(np.float32)
                bands.append(band_3)
            if which_bands[i] == 'band_4':
                band_4 = naip.GetRasterBand(4).ReadAsArray().astype(np.float32)
                bands.append(band_4)

        save_comp(bands, out_composite, naip)

    def NDVI(self, out_raster=None):
        """
        Calculates NDVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """

        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_1', 'band_4'])

        # Perform Calculation
        equation = ((bands['band_4'] - bands['band_1']) /
                    (bands['band_4'] + bands['band_1']))

        if out_raster is not None:
            save_index(equation, out_raster, bands['snap'])
            return equation
        if out_raster is None:
            return equation

    def ARVI(self, out_raster=None):
        """
        Calculates ARVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_1', 'band_3', 'band_4'])

        # Perform Calculation
        equation = ((bands['band_4'] - (2 * bands['band_1']) + bands['band_3']) /
                    (bands['band_4'] + (2 * bands['band_1']) + bands['band_3']))

        if out_raster is not None:
            save_index(equation, out_raster, bands['snap'])
            return equation
        if out_raster is None:
            return equation

    def VARI(self, out_raster=None):
        """
        Calculates VARI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_1', 'band_2', 'band_3'])

        # Perform Calculation
        equation = ((2 * bands['band_2'] - (bands['band_1'] + bands['band_3'])) /
                    (2 * bands['band_2'] + (bands['band_1'] + bands['band_3'])))

        if out_raster is not None:
            save_index(equation, out_raster, bands['snap'])
            return equation
        if out_raster is None:
            return equation

    def SAVI(self, soil_brightness=0.5, out_raster=None):
        """
        Calculates SAVI index

        Parameters
        ----------
            soil_brightness : float
                Soil brightness factor to compute SAVI with. Defaults to 0.5
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """
        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_1', 'band_4'])

        # Perform Calculation
        equation = (((bands['band_4'] - bands['band_1']) / (bands['band_4'] + bands['band_1'] + soil_brightness))
                    * (1 + soil_brightness))

        if out_raster is not None:
            save_index(equation, out_raster, bands['snap'])
            return equation
        if out_raster is None:
            return equation

    def RedRatio(self, out_raster=None):
        """
        Calculates ARVI index

        Parameters
        ----------
            out_raster : str, optional
                Output filepath for calculated TIFF.

        Returns
        -------
            equation : array
                Output array of the generated index.

        """

        gdal.PushErrorHandler('CPLQuietErrorHandler')
        gdal.UseExceptions()
        gdal.AllRegister()
        np.seterr(divide='ignore', invalid='ignore')

        bands = self.load_bands(['band_1', 'band_2', 'band_3'])
        # Perform Calculation
        equation = (bands['band_3'] + bands['band_1'] + bands['band_2']) / bands['band_1']

        if out_raster is not None:
            save_index(equation, out_raster, bands['snap'])
            return equation
        if out_raster is None:
            return equation



def norm(array, max_value, min_value):
    array_min, array_max = array.min(), array.max()
    return ((max_value - min_value) * ((array - array_min) /
            (array_max - array_min))) + min_value


from osgeo import gdal
import numpy as np


def gen_stats(raster_path):
    """
    Prints minimum, maximum, mean, median, and standard deviation values for
    a raster.

    Parameters
    ----------
        raster_path : str, required
            input raster with which to generate statistical summary of.

    Returns
    -------
        minimum
        maximum
        mean
        median
        standard deviation
    """

    path = gdal.Open(raster_path)
    input_arr = path.GetRasterBand(1).ReadAsArray().astype(np.float32)

    arr_min = np.nanmin(input_arr)
    arr_max = np.nanmax(input_arr)
    arr_mean = np.nanmean(input_arr)
    arr_med = np.nanmedian(input_arr)
    arr_std = np.nanstd(input_arr)
    print('min: ' + str(arr_min),
          'max: ' + str(arr_max),
          'mean: ' + str(arr_mean),
          'med: ' + str(arr_med),
          'std: ' + str(arr_std))
    return arr_min, arr_max, arr_mean, arr_med, arr_std
